﻿namespace HeuristicLab.Algorithms.PushGP.Expressions
{
    using HeuristicLab.Algorithms.PushGP.Generators;
    using HeuristicLab.Algorithms.PushGP.Interpreter;

    /// <summary>
    ///     Pushes a random NAME.
    /// </summary>
    public class NameRandExpression : StatelessExpression
    {
        public NameRandExpression() : base("NAME.RAND")
        {
        }

        public override void Eval(IPushGpInterpreter interpreter)
        {
            var name = NameGenerator.RandomName();

            interpreter.NameStack.Push(name);
        }
    }

    /// <summary>
    ///     Pushes a random integer.
    /// </summary>
    public class IntegerRandExpression : StatelessExpression
    {
        public IntegerRandExpression() : base("INTEGER.RAND")
        {
        }

        public override void Eval(IPushGpInterpreter interpreter)
        {
            var value = IntegerGenerator.RandomInteger(
                interpreter.Configuration.MinRandomInteger,
                interpreter.Configuration.MaxRandomInteger);

            interpreter.IntegerStack.Push(value);
        }
    }

    /// <summary>
    ///     Pushes a random float.
    /// </summary>
    public class FloatRandExpression : StatelessExpression
    {
        public FloatRandExpression() : base("FLOAT.RAND")
        {
        }

        public override void Eval(IPushGpInterpreter interpreter)
        {
            var value = FloatGenerator.RandomFloat(
                interpreter.Configuration.MinRandomFloat,
                interpreter.Configuration.MaxRandomFloat);

            interpreter.FloatStack.Push(value);
        }
    }

    /// <summary>
    ///     Pushes a random boolean.
    /// </summary>
    public class BooleanRandExpression : StatelessExpression
    {
        public BooleanRandExpression() : base("BOOLEAN.RAND")
        {
        }

        public override void Eval(IPushGpInterpreter interpreter)
        {
            var value = BooleanGenerator.RandomBoolean();

            interpreter.BooleanStack.Push(value);
        }
    }

    /// <summary>
    ///     Pushes random expressions onto the code stack.
    /// </summary>
    public class CodeRandExpression : StatelessExpression
    {
        public CodeRandExpression() : base("CODE.RAND")
        {
        }

        public override void Eval(IPushGpInterpreter interpreter)
        {
            if (interpreter.IntegerStack.Count == 0 ||
                interpreter.IntegerStack.Top < 1) return;

            var size = (int) (interpreter.IntegerStack.Pop() % interpreter.Configuration.MaxPointsInRandomExpression);
            var program = CodeGenerator.RandomProgram(size, interpreter.CustomExpressions);

            interpreter.CodeStack.Push(program);
        }
    }
}