﻿namespace HeuristicLab.Algorithms.PushGP.Expressions
{
    using Interpreter;
    using Stack;

    public abstract class RotateExpression<T> : StatelessExpression
    {
        protected RotateExpression(string stringRepresentation) : base(stringRepresentation) { }

        public void Eval(IStack<T> stack)
        {
            if (stack.Count < 3) return;

            stack.Swap(3);
        }
    }

    public class IntegerRotateExpression : RotateExpression<long>
    {
        public IntegerRotateExpression() : base("INTEGER.ROT") { }

        public override void Eval(IPushGpInterpreter interpreter)
        {
            Eval(interpreter.IntegerStack);
        }
    }

    public class FloatRotateExpression : RotateExpression<double>
    {
        public FloatRotateExpression() : base("FLOAT.ROT") { }

        public override void Eval(IPushGpInterpreter interpreter)
        {
            Eval(interpreter.FloatStack);
        }
    }

    public class BooleanRotateExpression : RotateExpression<bool>
    {
        public BooleanRotateExpression() : base("BOOLEAN.ROT") { }

        public override void Eval(IPushGpInterpreter interpreter)
        {
            Eval(interpreter.BooleanStack);
        }
    }

    public class NameRotateExpression : RotateExpression<string>
    {
        public NameRotateExpression() : base("NAME.ROT") { }

        public override void Eval(IPushGpInterpreter interpreter)
        {
            Eval(interpreter.NameStack);
        }
    }

    public class ExecRotateExpression : RotateExpression<Expression>
    {
        public ExecRotateExpression() : base("EXEC.ROT") { }

        public override void Eval(IPushGpInterpreter interpreter)
        {
            Eval(interpreter.ExecStack);
        }
    }

    public class CodeRotateExpression : RotateExpression<Expression>
    {
        public CodeRotateExpression() : base("CODE.ROT") { }

        public override void Eval(IPushGpInterpreter interpreter)
        {
            Eval(interpreter.CodeStack);
        }
    }
}